// Initializing enthalpy source term and adsorption heat in each time step
adsorptionHeat = 0.0 * adsorptionHeat;
enthalpySource = 0.0 * enthalpySource;

forAll(adsorbates, k)
{
    // Adsorption loading amount on the patch[Kg] 
    volScalarField& adsorption = adsorptions[k];

    // Equilibrium adsorption loading on the face[Kg/m2]
    volScalarField& eqAdsorption = eqAdsorptions[k];
    eqAdsorption = 0.0 * eqAdsorption;

    // Adsorption loading source-term[Kg/m3/s]
    volScalarField& volAdsorption = volAdsorptions[k];
    volAdsorption = 0.0 * volAdsorption;

    // Actual adsorption rate on the face[Kg/s]
    volScalarField& surfAdsorption = surfAdsorptions[k];
    surfAdsorption = 0.0 * surfAdsorption;

    //checks for matching adsorption and species fields
    label adsorpSpecie(-1);

    forAll(Y, l)
    {
        if (Y[l].name() == adsorbates[k])
        { 
            adsorpSpecie = l;
        }
    }

    forAll(solidRegions, j)
    {
        // Coupled patches fluid_to_solid
        label fluidSolidPatchID(fluidsSolidsPatchIDs[i][j]);

        // Adsorbent type ID
        label adsorbentTypeID(fluidsSolidsAdsorptionTypes[i][j]);

        if (fluidSolidPatchID > -1 && adsorbentTypeID > -1)
        {
            #include "setRegionAdsorptionFields.H"

            // Equilibrium constant temperature dependency
            volScalarField Kep = Ke / Foam::pow(T, np) * Foam::exp(-activationEnergy / R / T);

            if(Kep.dimensions() != dimensionSet(0, -1, 2, 0, 0, 0, 0))
            {
                Info << "\n\nWarning: Check Kep dimensions for " << adsorbates[k] 
                     << " at " << adsorbentTypes[adsorbentTypeID] << "!" << endl;
            } 

            // Calculating maximum adsorption on the patch
            forAll(currPatch, facei) //patch faces loop
            {
                label faceCelli = currPatch.faceCells()[facei];
                                
                const scalarField& pArea = area.boundaryField()[fluidSolidPatchID];
                
                scalarField& pP = p.boundaryField()[fluidSolidPatchID];
                scalarField& pKep = Kep.boundaryField()[fluidSolidPatchID];
                scalarField& pAdsorption = adsorption.boundaryField()[fluidSolidPatchID];
                scalarField& pEqAdsorption = eqAdsorption.boundaryField()[fluidSolidPatchID];
                scalarField& pSurfAdsorption = surfAdsorption.boundaryField()[fluidSolidPatchID];

                // Calculating mole fraction
                scalar M = 0;

                forAll(Y, l)
                {
                    M += Y[l].boundaryField()[fluidSolidPatchID][facei]/composition.W(l);
                }

                scalar xi 
                        = 
                          Y[adsorpSpecie].boundaryField()[fluidSolidPatchID][facei] 
                        / composition.W(adsorpSpecie)/M;

                // Equilibrium adsorption loading (according to Henry's law) on the face[Kg/m2]
                pEqAdsorption[facei] = pKep[facei] * pP[facei] * xi;

                // Checking for desorption for second order term
                scalar sign = 1;
                
                if((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]) < 0)
                {
                    sign = -1;
                }
                
                // Actual adsorption rate on the face[Kg/s]
                pSurfAdsorption[facei]
              = (
                K0.value() 
              + K1.value()*(pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]) 
              + K2.value()*Foam::pow((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]), 2)*sign
              + K3.value()*Foam::pow((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]), 3)
                )*Foam::pow((Cmax.value()-pEqAdsorption[facei])/Cmax.value(), n.value())*pArea[facei];
                
                // Actual adsorption loading in the cell [Kg]        
                maxAdsorption[faceCelli] 
             += pSurfAdsorption[facei]*deltaT;
            }

            // Calculating max available in the cell
            forAll(maxAvailable, celli) 
            {
                maxAvailable[celli] = Y[adsorpSpecie][celli] * rho[celli] * volume[celli];
            }

            // Calculating correction factor for available amount
            forAll(corrFact, celli)
            {
                if(maxAdsorption[celli] != 0.0)
                {
                    scalar ratio = maxAvailable[celli] / maxAdsorption[celli];
                    
                    if(ratio >= 1.0 || ratio < 0.0)
                        corrFact[celli] = 1;
                    else
                    {
                        corrFact[celli] = ratio;
                        counter++;
                    }
                }
            }

            // Calculating adsorption using correction factor
            forAll(currPatch, facei)
            {
                label faceCelli = currPatch.faceCells()[facei];

                const scalarField& pArea = area.boundaryField()[fluidSolidPatchID];
                
                scalarField& pAdsorption = adsorption.boundaryField()[fluidSolidPatchID];
                scalarField& pEqAdsorption = eqAdsorption.boundaryField()[fluidSolidPatchID];
                scalarField& pSurfAdsorption = surfAdsorption.boundaryField()[fluidSolidPatchID];
                scalarField& pAdsorptionHeat = adsorptionHeat.boundaryField()[fluidSolidPatchID];
                
                // Checking for desorption for second order term
                scalar sign = 1;
                
                if((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]) < 0)
                {
                    sign = -1;
                }
                
                // Corrected face adsorption rate[Kg/s]
                pSurfAdsorption[facei] 
              = (
                K0.value() 
              + K1.value()*(pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]) 
              + K2.value()*Foam::pow((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]), 2)*sign
              + K3.value()*Foam::pow((pEqAdsorption[facei]-pAdsorption[facei]/pArea[facei]), 3)
                )*Foam::pow((Cmax.value()-pEqAdsorption[facei])/Cmax.value(), n.value()) 
              * pArea[facei]*corrFact[faceCelli];
    
                // Adsorption loading source-term[Kg/m3/s]
                volAdsorption[faceCelli] 
             -= pSurfAdsorption[facei]/volume[faceCelli];

                // Updating adsorption loading amount on the patch[Kg]
                pAdsorption[facei] 
             += pSurfAdsorption[facei]*deltaT;

                // Calculating adsorption heat
                pAdsorptionHeat[facei]
             +=
                pSurfAdsorption[facei] * adsorptionDeltaH.value() * deltaT;

                // Integrating adsorption variables over domain
                totLoading += pAdsorption[facei];
                totEqLoading += pEqAdsorption[facei] * pArea[facei];
                totLoadingRate += pSurfAdsorption[facei];
                totAdsorptionArea += pArea[facei];
            }

            //collect across all processors
            reduce(counter, sumOp<scalar>());
            reduce(totLoading, sumOp<scalar>());
            reduce(totEqLoading, sumOp<scalar>());
            reduce(totLoadingRate, sumOp<scalar>());
            reduce(totAdsorptionArea, sumOp<scalar>());

            // The specie and the solid intraction
            bool boundarySpecieIntraction(boundariesSpeciesIntractions[adsorbentTypeID][k]);

            if (boundarySpecieIntraction)
            {
                Info<< "\nAdsorption data for '" << adsorbates[k] << "' at \"" << fluidRegions[i].name() 
                    << " to " << solidRegions[j].name() << "\" boundary:" << endl;
                Info<< "In " << counter << " cells adsorption limiter applied" << endl;
                Info<< "Total adsorption area = " << totAdsorptionArea << " m2" << endl;
                Info<< "Actual adsorption rate = " << totLoadingRate << " Kg/s" << endl;
                Info<< "Actual adsorption loading = " << totLoading << " Kg" << endl;
                Info<< "Equilibrium adsorption loading = " << totEqLoading << " Kg" << endl;
            }
        }  
    }

    // Calculating enthalpy source term caused by mass source term
    forAll(enthalpySource, celli)
    {
        enthalpySource[celli] 
      += 
        composition.Hs(adsorpSpecie, p[celli], T[celli]) * volAdsorption[celli];
    }
}