    // Initialise fluid field pointer lists
    PtrList<combustionModels::psiCombustionModel> chemistryFluid(fluidRegions.size());
    PtrList<volScalarField> rhoFluid(fluidRegions.size());
    PtrList<volVectorField> UFluid(fluidRegions.size());
    PtrList<volScalarField> TFluid(fluidRegions.size());
    PtrList<surfaceScalarField> phiFluid(fluidRegions.size());
    PtrList<uniformDimensionedVectorField> gFluid(fluidRegions.size());
    PtrList<compressible::turbulenceModel> turbulence(fluidRegions.size());
    PtrList<volScalarField> p_rghFluid(fluidRegions.size());
    PtrList<volScalarField> ghFluid(fluidRegions.size());
    PtrList<surfaceScalarField> ghfFluid(fluidRegions.size());
    PtrList<radiation::radiationModel> radiation(fluidRegions.size());
    PtrList<volScalarField> KFluid(fluidRegions.size());
    PtrList<volScalarField> dpdtFluid(fluidRegions.size());
    List<scalar> initialMassFluid(fluidRegions.size());
    PtrList<fv::IOoptionList> fluidFvOptions(fluidRegions.size());

    // Initialise adsorption field pointer lists
    PtrList<volScalarField> adsorptionHeatFluid(fluidRegions.size());
    PtrList<volScalarField> enthalpySourceFluid(fluidRegions.size());
    PtrList<PtrList<volScalarField> > adsorptionsFluid(fluidRegions.size());
    PtrList<PtrList<volScalarField> > eqAdsorptionsFluid(fluidRegions.size());
    PtrList<PtrList<volScalarField> > volAdsorptionsFluid(fluidRegions.size());
    PtrList<PtrList<volScalarField> > surfAdsorptionsFluid(fluidRegions.size());

    // Populate fluid field pointer lists
    forAll(fluidRegions, i)
    {
        Info<< "*** Reading fluid mesh thermophysical properties for region "
            << fluidRegions[i].name() << nl << endl;

        Info<< "    Adding to chemistryFluid\n" << endl;

        chemistryFluid.set
        (
            i,
            combustionModels::psiCombustionModel::New(fluidRegions[i]).ptr()
        );

        Info<< "    Adding to rhoFluid\n" << endl;
        rhoFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "rho",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                chemistryFluid[i].thermo().rho()
            )
        );

        Info<< "    Adding to UFluid\n" << endl;
        UFluid.set
        (
            i,
            new volVectorField
            (
                IOobject
                (
                    "U",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::AUTO_WRITE
                ),
                fluidRegions[i]
            )
        );

        Info<< "    Adding to phiFluid\n" << endl;
        phiFluid.set
        (
            i,
            new surfaceScalarField
            (
                IOobject
                (
                    "phi",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::READ_IF_PRESENT,
                    IOobject::AUTO_WRITE
                ),
                linearInterpolate(rhoFluid[i]*UFluid[i])
                    & fluidRegions[i].Sf()
            )
        );

        Info<< "    Adding to gFluid\n" << endl;
        gFluid.set
        (
            i,
            new uniformDimensionedVectorField
            (
                IOobject
                (
                    "g",
                    runTime.constant(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                )
            )
        );

        Info<< "    Adding turbulence \n" << endl;
        turbulence.set
        (
            i,
            compressible::turbulenceModel::New
            (
                rhoFluid[i],
                UFluid[i],
                phiFluid[i],
                chemistryFluid[i].thermo()
            ).ptr()
        );

        Info<< "    Adding to ghFluid\n" << endl;
        ghFluid.set
        (
            i,
            new volScalarField("gh", gFluid[i] & fluidRegions[i].C())
        );

        Info<< "    Adding to ghfFluid\n" << endl;
        ghfFluid.set
        (
            i,
            new surfaceScalarField("ghf", gFluid[i] & fluidRegions[i].Cf())
        );

        p_rghFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "p_rgh",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::AUTO_WRITE
                ),
                fluidRegions[i]
            )
        );

        // Force p_rgh to be consistent with p
        p_rghFluid[i] = chemistryFluid[i].thermo().p() - rhoFluid[i]*ghFluid[i];

        radiation.set
        (
            i,
            radiation::radiationModel::New(chemistryFluid[i].thermo().T())
        );

        initialMassFluid[i] = fvc::domainIntegrate(rhoFluid[i]).value();

        Info<< "    Adding to KFluid\n" << endl;
        KFluid.set
        (
            i,
            new volScalarField
            (
                "K",
                0.5*magSqr(UFluid[i])
            )
        );

        Info<< "    Adding to dpdtFluid\n" << endl;
        dpdtFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "dpdt",
                    runTime.timeName(),
                    fluidRegions[i]
                ),
                fluidRegions[i],
                dimensionedScalar
                (
                    "dpdt",
                    chemistryFluid[i].thermo().p().dimensions()/dimTime,
                    0
                )
            )
        );

        Info<< "    Adding fvOptions\n" << endl;
        fluidFvOptions.set
        (
            i,
            new fv::IOoptionList(fluidRegions[i])
        );

        // Creating adsorption correction source terms
        Info<< "    Adding to adsorptionHeat\n" << endl;
        adsorptionHeatFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "adsorptionHeat",
                    runTime.timeName(),
                    fluidRegions[i]
                ),
                fluidRegions[i],
                dimensionedScalar
                (
                    "adsorptionHeat", 
                    dimEnergy, 
                    0.0
                )
            )
        );

        Info<< "    Adding to enthalpySource\n" << endl;
        enthalpySourceFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "enthalpySource",
                    runTime.timeName(),
                    fluidRegions[i]
                ),
                fluidRegions[i],
                dimensionedScalar
                (
                    "enthalpySource", 
                    dimEnergy/dimVol/dimTime, 
                    0.0
                )
            )
        );

        Info<< "    Adding to adsorptions\n" << endl;
        adsorptionsFluid.set
        (
            i, 
            new PtrList<volScalarField>(adsorbates.size())
        );

        PtrList<volScalarField>& adsorptions = adsorptionsFluid[i];

        forAll (adsorptions, j)
        {
            adsorptions.set
            (
                j, 
                new volScalarField
                (
                    IOobject
                    (
                        IOobject::groupName("adsorption", adsorbates[j]),
                        runTime.timeName(),
                        fluidRegions[i],
                        IOobject::MUST_READ,
                        IOobject::AUTO_WRITE
                    ),
                    fluidRegions[i]
                )
            );
        }

        Info<< "    Adding to eqAdsorptions\n" << endl;
        eqAdsorptionsFluid.set
        (
            i, 
            new PtrList<volScalarField>(adsorbates.size())
        );

        PtrList<volScalarField>& eqAdsorptions = eqAdsorptionsFluid[i];

        forAll (eqAdsorptions, j)
        {
            eqAdsorptions.set
            (
                j, 
                new volScalarField
                (
                    IOobject
                    (
                        IOobject::groupName("eqAdsorption", adsorbates[j]),
                        runTime.timeName(),
                        fluidRegions[i],
                        IOobject::NO_READ,
                        IOobject::AUTO_WRITE
                    ),
                    fluidRegions[i],
                    dimensionedScalar(adsorbates[j], dimensionSet(1, -2, 0, 0, 0, 0, 0), 0.0)
                )
            );
        }

        Info<< "    Adding to volAdsorptions\n" << endl;
        volAdsorptionsFluid.set
        (
            i, 
            new PtrList<volScalarField>(adsorbates.size())
        );

        PtrList<volScalarField>& volAdsorptions = volAdsorptionsFluid[i];

        forAll (volAdsorptions, j)
        {
            volAdsorptions.set
            (
                j, 
                new volScalarField
                (
                    IOobject
                    (
                        IOobject::groupName("volAdsorption", adsorbates[j]),
                        runTime.timeName(),
                        fluidRegions[i]
                    ),
                    fluidRegions[i],
                    dimensionedScalar(adsorbates[j], dimMass/dimVol/dimTime, 0.0)
                )
            );
        }

        Info<< "    Adding to surfAdsorptions\n" << endl;
        surfAdsorptionsFluid.set
        (
            i, 
            new PtrList<volScalarField>(adsorbates.size())
        );

        PtrList<volScalarField>& surfAdsorptions = surfAdsorptionsFluid[i];

        forAll (surfAdsorptions, j)
        {
            surfAdsorptions.set
            (
                j, 
                new volScalarField
                (
                    IOobject
                    (
                        IOobject::groupName("surfAdsorption", adsorbates[j]),
                        runTime.timeName(),
                        fluidRegions[i],
                        IOobject::NO_READ,
                        IOobject::AUTO_WRITE
                    ),
                    fluidRegions[i],
                    dimensionedScalar(adsorbates[j], dimMass/dimTime, 0.0)
                )
            );
        }
    }