//accessing adsorbents properties dictionary
IOdictionary adsorptionDict
(
    IOobject
    (
        "adsorbentProperties",
        runTime.time().constant(),
        runTime.db(),
        IOobject::MUST_READ_IF_MODIFIED,
        IOobject::NO_WRITE
    )
);

//reading adsorbate species list
wordList adsorbates
(
    adsorptionDict.lookup("adsorbates")
);

//reading adsorbents list
wordList adsorbentTypes
(
    adsorptionDict.lookup("adsorbentTypes")
);

//reading adsorbents parameters for different species
Info<< "Reading adsorbents properties\n" << endl;

PtrList< PtrList<bool> > boundariesSpeciesIntractions(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsNs(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsNps(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsCmaxs(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsK0s(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsK1s(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsK2s(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsK3s(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorbentsKes(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorptionDeltaHs(adsorbentTypes.size());
PtrList< PtrList<dimensionedScalar> > adsorptionActivationEnergies(adsorbentTypes.size());

forAll(adsorbentTypes, i)
{
    boundariesSpeciesIntractions.set
    (
        i, 
        new PtrList<bool>(adsorbates.size())
    );

    adsorbentsNs.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsNps.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsCmaxs.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsK0s.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsK1s.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsK2s.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsK3s.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorbentsKes.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorptionDeltaHs.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );

    adsorptionActivationEnergies.set
    (
        i, 
        new PtrList<dimensionedScalar>(adsorbates.size())
    );


    PtrList<bool>& boundarySpeciesIntractions = boundariesSpeciesIntractions[i];
    PtrList<dimensionedScalar>& ns = adsorbentsNs[i];
    PtrList<dimensionedScalar>& nps = adsorbentsNps[i];
    PtrList<dimensionedScalar>& Cmaxs = adsorbentsCmaxs[i];
    PtrList<dimensionedScalar>& K0s = adsorbentsK0s[i];
    PtrList<dimensionedScalar>& K1s = adsorbentsK1s[i];
    PtrList<dimensionedScalar>& K2s = adsorbentsK2s[i];
    PtrList<dimensionedScalar>& K3s = adsorbentsK3s[i];
    PtrList<dimensionedScalar>& Kes = adsorbentsKes[i];
    PtrList<dimensionedScalar>& deltaHs = adsorptionDeltaHs[i];
    PtrList<dimensionedScalar>& activationEnergies = adsorptionActivationEnergies[i];


    forAll(adsorbates, j)
    {
        const word& adsorbateName = adsorbates[j];
        const word& adsorbentType = adsorbentTypes[i];
        const dictionary& subDict = adsorptionDict.subDict(adsorbentType);

        // Checks if the subdict exists and read from it or use default values
        if (subDict.isDict(adsorbateName))
        {
            const dictionary& subSubDict = subDict.subDict(adsorbateName);

            boundarySpeciesIntractions.set
            (
                 j,
                 new bool(true)
            );

            ns.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("n", 0.0))
            );

            nps.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("np", 0.0))
            );

            Cmaxs.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("Cmax", 1.0))
            );

            K0s.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("K0", 0.0))
            );

            K1s.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("K1", 0.0))
            );

            K2s.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("K2", 0.0))
            );

            K3s.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("K3", 0.0))
            );

            Kes.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("Ke", 0.0))
            );

            deltaHs.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("deltaH", 0.0))
            );

            activationEnergies.set
            (
                 j,
                 new dimensionedScalar(subSubDict.lookupOrDefault<dimensionedScalar>("activationEnergy", 0.0))
            );
        }
        // Settind default values if the dictionary is not present
        else
        {
            boundarySpeciesIntractions.set
            (
                 j,
                 new bool(false)
            );

            ns.set
            (
                 j,
                 new dimensionedScalar("n", dimensionSet(0, 0, 0, 0, 0, 0, 0), 0.0)
            );

            nps.set
            (
                 j,
                 new dimensionedScalar("np", dimensionSet(0, 0, 0, 0, 0, 0, 0), 0.0)
            );

            Cmaxs.set
            (
                 j,
                 new dimensionedScalar("Cmax", dimensionSet(1, -2, 0, 0, 0, 0, 0), 1.0)
            );

            K0s.set
            (
                 j,
                 new dimensionedScalar("K0", dimensionSet(1, -2, -1, 0, 0, 0, 0), 0.0)
            );

            K1s.set
            (
                 j,
                 new dimensionedScalar("K1", dimensionSet(0, 0, -1, 0, 0, 0, 0), 0.0)
            );

            K2s.set
            (
                 j,
                 new dimensionedScalar("K2", dimensionSet(-1, 2, -1, 0, 0, 0, 0), 0.0)
            );

            K3s.set
            (
                 j,
                 new dimensionedScalar("K3", dimensionSet(-2, 4, -1, 0, 0, 0, 0), 0.0)
            );

            Kes.set
            (
                 j,
                 new dimensionedScalar("Ke", dimensionSet(0, -1, 2, 0, 0, 0, 0), 0.0)
            );

            deltaHs.set
            (
                 j,
                 new dimensionedScalar("deltaH", dimensionSet(0, 2, -2, 0, 0, 0, 0), 0.0)
            );

            activationEnergies.set
            (
                 j,
                 new dimensionedScalar("activationEnergy", dimensionSet(1, 2, -2, 0, -1, 0, 0), 0.0)
            );
        }
    }
}

