adsorptionHeat = 0.0 * adsorptionHeat;

forAll(fluidRegions, j)
{
    const fvPatchList& patches = mesh.boundary();
    const scalarField& volume = mesh.V();
    const scalar& deltaT = runTime.deltaTValue();

    // Find coupled patches solid_to_fluid
    label solidFluidPatchID = mesh.boundaryMesh().findPatchID
    (
        mesh.name() + "_to_" + fluidRegions[j].name()
    );

    label fluidSolidPatchID = fluidRegions[j].boundaryMesh().findPatchID
    (
        fluidRegions[j].name() + "_to_" + mesh.name()
    );

    if (solidFluidPatchID > -1)
    {
        volScalarField neiAdsorptionHeat = 0.0 * adsorptionHeat;

        // Reading adsorption heat field from neighbour region
        const mappedPatchBase& mpp = refCast<const mappedPatchBase>
        (
            adsorptionHeat.boundaryField()[solidFluidPatchID].patch().patch()
        );

        // Force recalculation of mapping and schedule
        const mapDistribute& distMap = mpp.map();
        
        neiAdsorptionHeat.boundaryField()[solidFluidPatchID] 
      == 
        UFluid[j].db().lookupObject<volScalarField>
        ("adsorptionHeat").boundaryField()[fluidSolidPatchID];

        // Swap to obtain full local values of neighbour field
        mapDistribute::distribute
        (
            Pstream::defaultCommsType,
            distMap.schedule(),
            distMap.constructSize(),
            distMap.subMap(),
            distMap.constructMap(),
            neiAdsorptionHeat.boundaryField()[solidFluidPatchID]
        );

        const fvPatch& currPatch = patches[solidFluidPatchID];

        forAll(currPatch, facei)
        {
            label faceCelli = currPatch.faceCells()[facei];
            scalarField& pNeiAdsorptionHeat = neiAdsorptionHeat.boundaryField()[solidFluidPatchID];

            adsorptionHeat[faceCelli]
         =
            pNeiAdsorptionHeat[facei] / volume[faceCelli] / deltaT;
        }
    }
}

